package com.test;

import java.io.StringReader;
import java.util.Date;
import java.util.concurrent.TimeUnit;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.conf.EventProcessingOption;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
//import org.drools.logger.KnowledgeRuntimeLogger;
//import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.conf.ClockTypeOption;
import org.drools.runtime.rule.WorkingMemoryEntryPoint;
import org.drools.time.SessionPseudoClock;


public class DroolsTest {

    public class Foo implements java.io.Serializable {
	private Date date = null;

	public Foo(Date date) {
	    this.date = date;
	}

	public Date getDate() {
	    return this.date;
	}

	public void setDate(Date date) {
	    this.date = date;
	}
    }

    private StatefulKnowledgeSession ksession = null;
    private WorkingMemoryEntryPoint cds = null;
    private String drl = 
	"import com.test.DroolsTest.Foo " +
        "declare Foo " +
        "	@role ( event ) " +
        "	@expires ( 60s ) " +
        "	@timestamp ( date ) " +
        "end " +
    
        "rule \"Count Foo\" " +
        "when " +
        "	c : Number(intValue > 5) from accumulate ( " +
        "		$f : Foo() " +
        "		over window:time ( 30s ) " +
        "		from entry-point ChannelDataStream, " +
        "	count($f)) " +
        "then " +
        "	System.out.println(\"Count [30s] = \" + c); " +
        "end ";

    public DroolsTest() {

	try {
	    KnowledgeBuilder kbuilder = KnowledgeBuilderFactory
		    .newKnowledgeBuilder();

	    kbuilder
		    .add(ResourceFactory
			    .newReaderResource(new StringReader(drl)),
			    ResourceType.DRL);

	    KnowledgeBuilderErrors errors = kbuilder.getErrors();

	    if (errors.size() > 0) {
		for (KnowledgeBuilderError error : errors) {
		    System.err.println(error);
		}
		throw new IllegalArgumentException("Could not parse knowledge.");
	    }

	    KnowledgeBaseConfiguration config = KnowledgeBaseFactory
		    .newKnowledgeBaseConfiguration();
	    config.setOption(EventProcessingOption.STREAM);

	    KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase(config);
	    kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());

	    KnowledgeSessionConfiguration sconfig = KnowledgeBaseFactory
		    .newKnowledgeSessionConfiguration();
	    sconfig.setOption(ClockTypeOption.get("pseudo"));

	    ksession = kbase.newStatefulKnowledgeSession(sconfig, null);
	    
	    KnowledgeRuntimeLoggerFactory.newConsoleLogger(ksession);

	    cds = ksession.getWorkingMemoryEntryPoint("ChannelDataStream");


	    try {
		SessionPseudoClock clock = ksession.getSessionClock();
		
		// insert one Foo() object every 5 seconds.
		for (int i = 0; i < 15; i++) {
		    System.out.println("Inserted event " + (i + 1));
		    cds.insert(new Foo(new Date(clock.getCurrentTime())));
		    
		    clock.advanceTime(5, TimeUnit.SECONDS);
		    
		    ksession.fireAllRules();
		}

	    } catch (Exception t) {
		t.printStackTrace();
	    }

	} catch (Throwable t) {
	    t.printStackTrace();
	}
    }
    
    public static final void main(String[] args) {
	DroolsTest t = new DroolsTest();
    }
}